#!/usr/bin/php
<?php
/**
 * fsl2006 congress
 * topic: PHP router in Asterisk
 * author: Moises Silva <moy@ivsol.net>
 * @package fsl-astrouter
 */

/**
 * complete and simple AGI router.
 */
require_once 'AgiConnector.php';
require_once 'RouteHoncho.php';
/* create a new connection with Asterisk */
$connector     = new AgiConnector();
$connector->Write('Starting Router');
$connector->Write('user has dialed: ' . $connector->GetAgiVariable('extension'));

try
{
	/* create a new instance of the expert in routing */
	$route_honcho  = new RouteHoncho();  

	/* get the number dialed by the user */
	$dialed_number = $connector->GetAgiVariable('extension');

	/* get whos calling */
	$callerid      = $connector->GetAgiVariable('callerid');

	/* get where wants to call */
	$mapping       = $route_honcho->GetNumberMapping($dialed_number);
	if ( NULL === $mapping )
	{
		$connector->Playback('iss_invalid_pbx_number_en');
		exit;
	}
	/* is the extension authorized to make this call? */
	$try_times = 0;
	$pin       = NULL;
	do
	{
		if ( $route_honcho->CallIsAuthorized($callerid, $mapping['map_id'], $pin) )
		{
			/* this can be much, much better passing control to "Application Drivers"
			 * instead of having hard coded Applications
			 * */
			switch ( $mapping['name'] )
			{
				case RouteHoncho::APPLICATION_DIAL:
					$connector->Dial($mapping['data'] . '/' . $dialed_number);
					break;

				case RouteHoncho::APPLICATION_VOICEMAIL:
					$connector->EnterVoiceMail($dialed_number);
					break;
					
				case RouteHoncho::APPLICATION_CONFERENCE:
					$connector->StartConference($dialed_number);
					break;

				default:
					$connector->Playback('iss_hangup_en');
					break;
			}
			exit;
		}
		/* is not the first time, then play sound to indicate incorrect auth PIN */
		if ( $try_times > 0 )
		{
			$connector->Playback('iss_wrong_auth_en');
		}
		/* retrieve DTMF ddigits from the user asking for PIN */
		$pin = $connector->GetDigitsFromUser('iss_ask_auth_en');
		$connector->Write('user has pressed: ' . $pin);
		$try_times++;
	}	
	while ( $try_times <= 3 );
	$connector->Playback('iss_hangup_en');
}
catch ( Exception $error )
{
	/* ooops something has gone wrong, try to play technical failure sound */
	$connector->Write($error->getMessage());
	$connector->Playback('iss_technical_failure_en');
}
?>
