
inherit eutils 

IUSE="alsa bri curl debug doc gtk h323 hardened lowmem mmx mysql elibc_uclibc \
	inosamples odbc postgres pri speex sqlite ssl ukcid zaptel magi
	codec-negotiation"

BRI_VERSION="0.3.0-PRE-1"
AST_PATCHES="1.2.0-patches-1.1"

DESCRIPTION="Asterisk Intervoice Solutions Version: A Modular Open Source PBX System"
HOMEPAGE="http://www.asterisk.org/ http://www.ivsol.net"
SRC_URI="http://ftp.digium.com/pub/asterisk/asterisk-1.2.1.tar.gz
	 http://www.netdomination.org/pub/asterisk/asterisk-${AST_PATCHES}.tar.bz2
	 bri? ( http://www.junghanns.net/downloads/bristuff-${BRI_VERSION}.tar.gz )"

S="${WORKDIR}/${MY_P}"

S_BRI="${WORKDIR}/bristuff-${BRI_VERSION}"

SLOT="0"
LICENSE="GPL-2"
KEYWORDS="x86"

RDEPEND="dev-libs/newt
	media-sound/sox
	media-sound/mpg123
	alsa? ( media-libs/alsa-lib )
	speex? ( media-libs/speex )"

DEPEND="${RDEPEND}
	>=net-misc/zaptel-iss-1.2.1
	>=net-libs/libpri-iss-1.2.1
    >=net-libs/zapata-iss-1.0.1
	>=dev-db/unixODBC-2.2.11-r1
	>sys-process/daemontools-0.76-r4
	>=dev-db/sqlite-2.8.15
	sys-devel/flex
	sys-devel/bison"

pkg_setup(){
	intervoice_start_message
}

src_unpack() {
	unpack ${A}
	cd asterisk-1.2.1

	#
	# gentoo patchset
	#
	for x in $(grep -v "^#\| \+" ${FILESDIR}/ivsol-modified-patches.list)
	do
		epatch ${WORKDIR}/patches/${x}
	done
	
	if use mmx; then
		if ! use hardened; then
			einfo "Enabling mmx optimization"
			sed -i -e "s:^#\(K6OPT[\t ]\+= -DK6OPT\):\1:" \
				Makefile
		else
			ewarn "Hardened use-flag is set, not enabling mmx optimization for codec_gsm!"
		fi
	fi

	if ! use debug; then
		einfo "Disabling debug support"
		sed -i -e "s:^\(DEBUG=\):#\1:" \
			Makefile
	fi

	#
	# uclibc patch
	#
	if use elibc_uclibc; then
		einfo "Patching asterisk for uclibc..."
		epatch ${FILESDIR}/1.0.0/asterisk-1.0.5-uclibc-dns.diff
	fi

	# gave me some compile errors on zapata.c (moy@ivsol.net)
	# ukcid patch from http://www.lusyn.com/asterisk/
	#
	#if use ukcid; then
	#	einfo "Patching asterisk for UK Callerid..."
	#	epatch ${FILESDIR}/1.2.0/asterisk-1.2.0_beta-ukcid.patch
	#fi

	#
	# BRI patches
	#
	if use bri; then
		einfo "Patching asterisk w/ BRI stuff"
		epatch ${S_BRI}/patches/asterisk.patch
	fi
	
	if use magi
	then
		#
		# MAGI patch
		#
		epatch ${FILESDIR}/asterisk-1.2.1-magi_support.patch
	fi

	#only experimental
	if use codec-negotiation
	then
		#
		# Codec negotiation patch
		#
		PATCH_OUTPUT="/var/tmp/portage/asterisk-iss-1.2.1/temp/codec_negotiation-20051107-ivsol0.out"
		patch -p0 --ignore-whitespace <	${FILESDIR}/codec_negotiation-20051107-ivsol0.patch 1>/dev/null 2>${PATCH_OUTPUT} ||
		die "couldn't apply patch ${FILESDIR}/codec_negotiation-20051107-ivsol0.patch!"
		einfo "patch for codec_negotiation applied successfully"
		epatch ${FILESDIR}/asterisk-1.2.1-silence-suppression-4.patch
	fi

	# fix show channels concise to parse the output without problems
	epatch ${FILESDIR}/asterisk-1.2.1-show-channels-concise.patch

	# add functionality to meetme list <confno> concise
	epatch ${FILESDIR}/asterisk-1.2.1-meetme-list-concise.patch

    # avoid deleting .call files when timestamp is in the future
	# IMPORTANT for correct behaviour of Appointment (virtual secretary) pavoz application
	epatch ${FILESDIR}/asterisk-1.2.1-pbx_spool-fix.patch

	# IMPORTANT for correct behaviour of attended transfer
	epatch ${FILESDIR}/asterisk-1.2.1-res_features-fix.patch
	epatch ${FILESDIR}/asterisk-1.2.1-app_dial-peer-vars.patch

	# IMPORTANT for the correct behaviour of CallMonitor
	epatch ${FILESDIR}/asterisk-1.2.1-chanspy-complete-channel-fix.patch
			
    # enable intervoice email user number feature
    # and fix a NULL reference bug
    epatch ${FILESDIR}/asterisk-1.2.1-app_voicemail-ivsol-fixes.patch
				
	# Enable Sqlite CDR userfield
	cd ${WORKDIR}/asterisk-1.2.1/cdr/
	epatch ${FILESDIR}/sqlitecdr-userfield-enable.patch

}

src_compile() {
	cd asterisk-1.2.1
	local myopts

	use lowmem && \
		myopts="-DLOW_MEMORY"

	if use h323; then
		einfo "Building H.323 wrapper lib..."
		make -C channels/h323 \
			NOTRACE=1 \
			PWLIBDIR=/usr/share/pwlib \
			OPENH323DIR=/usr/share/openh323 \
			libchanh323.a Makefile.ast || die "Make h323 failed"
	fi
	einfo "Building Asterisk in ${dir}..."
	make \
		NOTRACE=1 \
		OPTIMIZE="${CFLAGS}" \
		PWLIBDIR=/usr/share/pwlib \
		OPENH323DIR=/usr/share/openh323 \
		OPTIONS="${myopts}" || die "Make failed"

	# create api docs
	use doc && \
		make progdocs

	# build bristuff's ISDNguard
	use bri && \
		make -C ${S_BRI}/ISDNguard
}

src_install() {

	# install asterisk
	cd asterisk-1.2.1	
	make DESTDIR=${D} install || die "Make install failed"
	make DESTDIR=${D} samples || die "Failed to create sample files"

	# remove installed sample files if nosamples flag is set
	if use nosamples; then
		einfo "Skipping installation of sample files..."
		einfo "See ${ROOT}usr/share/doc/${PF}/configs for configuration files"
		insinto /usr/share/doc/${PF}/configs
		newins  ${D}etc/asterisk/asterisk.conf asterisk.conf.sample

		# skip installation of sample configuration files
		# if asterisk-1.1.0 or later is present
		if has_version ">net-misc/asterisk-1.1.0"
		then
			rm -f  ${D}etc/asterisk/*
		else
			einfo "No previous or old (<=1.0.x) installation of ${PN} found,"
			einfo "installing sample configuration files!"
		fi
		rm -rf ${D}var/spool/asterisk/voicemail/default
		rm -f  ${D}var/lib/asterisk/mohmp3/*
		rm -f  ${D}var/lib/asterisk/sounds/demo-*
		rm -f  ${D}var/lib/asterisk/agi-bin/*
	else
		einfo "Sample files have been installed"
		keepdir /var/spool/asterisk/voicemail/default/1234/INBOX
	fi

	# don't delete these directories, even if they are empty
	for x in voicemail meetme system dictate tmp; do
		keepdir /var/spool/asterisk/${x}
	done
	keepdir /var/lib/asterisk/sounds/priv-callerintros
	keepdir /var/lib/asterisk/mohmp3
	keepdir /var/lib/asterisk/agi-bin
	keepdir /var/log/asterisk/cdr-csv
	keepdir /var/log/asterisk/cdr-custom
	keepdir /var/run/asterisk

	# install asterisk.h, a lot of external modules need this
	insinto /usr/include/asterisk
	doins	include/asterisk.h

	# install astgenkey, astxs, safe_asterisk and manpages
	dobin  contrib/scripts/astxs
	dosbin contrib/scripts/astgenkey
	dosbin contrib/scripts/safe_asterisk
	doman contrib/scripts/safe_asterisk.8
	doman contrib/scripts/astgenkey.8

	newinitd ${FILESDIR}/1.0.0/asterisk.rc6.sec asterisk
	newconfd ${FILESDIR}/1.0.0/asterisk.confd.sec asterisk

	# install standard docs...
	dodoc BUGS CREDITS LICENSE ChangeLog HARDWARE README README.fpm
	dodoc SECURITY doc/CODING-GUIDELINES doc/linkedlists.README UPGRADE.txt
	dodoc doc/README.*
	dodoc doc/*.txt

	docinto scripts
	dodoc contrib/scripts/*

	docinto utils
	dodoc contrib/utils/*

	docinto configs
	dodoc configs/*

	# install api docs
	if use doc; then
		insinto /usr/share/doc/${PF}/api/html
		doins doc/api/html/*
	fi

	# install ISDNguard
	if use bri; then
		cd ${S_BRI}/ISDNguard
		dosbin ISDNguard

		docinto ISDNguard
		dodoc INSTALL.ISDNguard

		cd ${S}
	fi

	insinto /usr/share/doc/${PF}/cgi
	doins contrib/scripts/vmail.cgi
	doins images/*.gif

    # creates daemontools config for asterisk
    if [ -d "/etc/svc/asterisk/" ]; then
        rm -r /etc/svc/asterisk/
    fi
    
    dodir /etc/svc/
    dodir /etc/svc/asterisk/
    dodir /etc/svc/asterisk/env/
    dodir /etc/svc/asterisk/log/
    dodir /etc/svc/asterisk/main/
    dodir /etc/svc/asterisk/supervise/
    
    # copy the start and log script
    exeinto /etc/svc/asterisk/
    newexe ${FILESDIR}/svc/run-asterisk-iss-1.2.1 run
    exeinto /etc/svc/asterisk/log/
    newexe ${FILESDIR}/svc/log-asterisk-iss-1.2.1 run

}

pkg_preinst() {
	enewgroup asterisk
	enewuser asterisk -1 -1 /var/lib/asterisk "asterisk,dialout"
}

pkg_postinst() {
	einfo "Fixing permissions"
	for x in spool run lib log; do
		chown -R asterisk:asterisk ${ROOT}var/${x}/asterisk
		chmod -R u=rwX,g=rX,o=     ${ROOT}var/${x}/asterisk
	done

	chown -R root:asterisk ${ROOT}etc/asterisk
	chmod -R u=rwX,g=rX,o= ${ROOT}etc/asterisk
	echo

	#
	# Announcements, warnings, reminders...
	#
	einfo "Asterisk has been installed"
	einfo ""
	einfo "If you want to know more about asterisk, visit these sites:"
	einfo "http://www.asteriskdocs.org/"
	einfo "http://www.voip-info.org/wiki-Asterisk"
	echo
	einfo "http://asterisk.xvoip.com/"
	einfo "http://junghanns.net/asterisk/"
	einfo "http://www.automated.it/guidetoasterisk.htm"
	echo
	einfo "Gentoo VoIP IRC Channel:"
	einfo "#gentoo-voip @ irc.freenode.net"
	echo
	echo

	#
	# Warning about security changes...
	#
	ewarn "                      Asterisk UPGRADE Warning"
	ewarn ""
	ewarn "!!! Read ${ROOT}usr/share/doc/${PF}/UPGRADE.txt.gz before continuing !!!"
	ewarn ""
	ewarn "                      Asterisk UPGRADE Warning"
}

pkg_config() {
	einfo "Do you want to reset file permissions and ownerships (y/N)?"

	read tmp
	tmp="$(echo $tmp | tr [:upper:] [:lower:])"

	if [[ "$tmp" = "y" ]] ||\
	   [[ "$tmp" = "yes" ]]
	then
		einfo "Resetting permissions to defaults..."

		for x in spool run lib log; do
			chown -R asterisk:asterisk ${ROOT}var/${x}/asterisk
			chmod -R u=rwX,g=rX,o=     ${ROOT}var/${x}/asterisk
		done

		chown -R root:asterisk ${ROOT}etc/asterisk
		chmod -R u=rwX,g=rX,o= ${ROOT}etc/asterisk

		einfo "done"
	else
		einfo "skipping"
	fi
}
